# -*- coding: latin-1 -*-
#    QUENLIG: Questionnaire en ligne (Online interactive tutorial)
#    Copyright (C) 2005-2011 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
#

from questions import *
from check import *

dumb_replace = (
    ('-re ', '-r '), ('-er ', '-r '),
    (' -e ', ' '), ('g; s', 'g;s'), ('g ; s', 'g;s'),
    )

add(name="intro",
    required=["manuel:chercher"],
    before="""La commande <tt>sed</tt> permet d'diter
    de trs gros fichiers sans les charger en mmoire.
    Par dfaut le fichier est lu sur l'entre standard
    et la nouvelle version est crite sur la sortie standard.
    La syntaxe de base est la substitution&nbsp;:
    <pre>sed -e 's/ancien texte/nouveau texte/g'</pre>
    Le <tt>g</tt> de la fin indique que l'on remplace
    toutes les occurrences qui sont dans la ligne
    et pas seulement la premire.""",
    question="""Donner la ligne de commande permettant de remplacer
    tous les <tt>blue</tt> de l'entre standard
    par des <tt>black</tt> sur la sortie standard.""",
    tests=(
    require('/g',
            """Si vous n'indiquez pas <tt>/g</tt>
            seul la premire occurrence dans la ligne sera substitue"""),
    require('/blue/', """Je ne vois nulle part de <tt>/blue/</tt>"""),
    require('/black/', """Je ne vois nulle part de <tt>/black/</tt>"""),
    require('s/',
            """Je ne vois nulle part le <tt>s/</tt> qui indique
            que l'on veut faire une substitution"""),
    shell_good("sed 's/blue/black/g'", dumb_replace=dumb_replace),
    number_of_is('/',3, """Si vous respectez la syntaxe, il devrait
    y avoir trois / dans la commande"""),
    shell_display,
    ),
    )

add(name="slash",
    required=["intro", "intro:back slash"],
    before="""Le caractre d'chappement utilis dans les commandes
    <tt>sed</tt> est le mme que celui utilis dans le shell,
    dans le langage C, dans les expressions rgulires, ...""",
    question="""Donner la ligne de commande permettant de remplacer
    tous les <tt>/</tt> de l'entre standard
    par des <tt>:</tt> (deux points) sur la sortie standard.""",
    tests=(
    require('/g', """N'oubliez pas le <tt>/g</tt> !"""),
    reject("s///",
              """Vous n'avez pas essay la commande....
              <tt>sed</tt> fait une erreur car il croit qu'on
              lui demande de remplace 'vide' par 'vide'
              et aprs il y a des caractres qu'il ne comprend pas.
              <p>
              Il faut banaliser le caractre <tt>/</tt> que vous voulez
              remplacer.
              """),
    shell_good(r"sed 's/\//:/g'",
               "L'autre solution tait <tt>[/]</tt> au lieu de <tt>\\/</tt>",
               dumb_replace=dumb_replace),
    shell_good(r"sed 's/[/]/:/g'",
               "L'autre solution tait <tt>\\/</tt> au lieu de <tt>[/]</tt>",
               dumb_replace=dumb_replace),
    require('\\', "Je ne vois pas le caractre d'chappement ncessaire"),
    reject('-r', "On a pas besoin d'expression rgulire tendue"),
    number_of_is('/', 4,
                 """Votre rponse doit contenir 4 slashs.
                 3 sont obligatoires pour faire une substitution normale
                 et il en faut un de plus car on veut remplacer
                 le slash par autre choses."""),
    require(':', "On veut mettre un ':' (deux point)  la place du slash"),
    reject('\\:', "Le ':' n'est pas spcial, pas la peine de le protger"),
    shell_display,
    ),
    indices=(
    """Vous ne trouverez pas la solution en multipliant les guillemets
    ou les apostrophes car le <tt>/</tt> n'est pas un caractre
    spcial pour le shell mais pour la commande <tt>sed</tt>""",
    "Le caractre d'chappement est \\",
    ),
    )

add(name="dans fichier",
    required=["intro", "sh:'Bonjour' dans 'toto'", "sh:redirection entre"],
    question="""Donnez la commande prenant le contenu du
    fichier <tt>xxx</tt>, remplaant tous les <tt>a</tt> par des <tt>b</tt>
    et stockant le rsultat dans le fichier <tt>yyy</tt>""",
    tests=(
    reject(('tr', 'cat'),
           """La seule commande qui vous est ncessaire est <tt>sed</tt>"""),
    require('sed', """Il faut utiliser <tt>sed</tt> pour la substitution"""),
    require('s/a/b/g',
            """Il faut indiquer  <tt>sed</tt>
            que l'on veux remplacer tous les <tt>a</tt> par des <tt>b</tt>"""),
    shell_good(("sed 's/a/b/g' <xxx >yyy",
                "sed 's/a/b/g' xxx >yyy") , dumb_replace=dumb_replace),
    require(('<', '>'),
            """Il faut rediriger l'entre
            et la sortie standard de <tt>sed</tt>"""),
    reject("-r", " quoi sert le <tt>-r</tt>&nbsp;?"),
    shell_display,
    ),
    indices=("Il faut rediriger l'entre et la sortie standard",),
    )

add(name="enlve commentaires",
    required=["dans fichier", "slash", "expreg:intro"],
    question="""On veut enlever tous les commentaires dfinis
    par <tt>// commentaire jusqu' la fin de la ligne</tt>
    du fichier <tt>tp.c</tt> et stocker le rsultat dans
    <tt>tp_.c</tt>
    <p>
    Si <tt>tp.c</tt> contient :
    <pre>jhsla sadf dsafsd afds fds f // sfasdf sdfasfas dsfa
saddasd asfddsafa //
//afsfs
a//b//c</pre>
Alors <tt>tp_.c</tt> contiendra :
<pre>jhsla sadf dsafsd afds fds f 
saddasd asfddsafa 

a</pre>
    """,
    tests=(
    require(".*",
            """Vous n'avez pas indiqu le <em>pattern</em> reprsentant
            une chaine de caractre quelconque pour indiquer
            ce qu'il y a aprs le <tt>//</tt>."""),
    reject("^", "Un commentaire ne commence pas forcment en dbut de ligne"),
    reject("/g", """L'option <tt>g</tt> de <tt>sed</tt>
    est inutile car il n'y a qu'une seule substitution  faire dans
    la ligne"""),
    reject("$", """Le <tt>$</tt> est inutile car le <tt>.*</tt>
    prend le maximum de caractres possible,
    il va donc jusqu' la fin de la ligne"""),    
    shell_good(r"sed 's/\/\/.*//' tp.c >tp_.c", dumb_replace=dumb_replace),
    shell_good(r"sed 's/\/\/.*//' <tp.c >tp_.c", dumb_replace=dumb_replace),
    require('\\/\\/',
            """Je ne vois pas les 2 <em>slashs</em> du commentaire
            dans l'expression.
            Avez-vous oubli de les protger
            avec un <em>anti-slash</em>&nbsp;?"""),
    require('//', """Il faut indiquer que vous remplacez le commentaire
    par une chaine vide (un espace n'est pas une chaine vide)"""),
    reject('-r',
           """Vous n'avez pas besoin d'expression rgulire tendue,
           enlevez les <tt>-r</tt>"""),
    require(("tp.c", "tp_.c"),
            """Je ne trouve pas les 2 noms de fichiers dans la commande"""),
    require('>',
            """Il faut rediriger la sortie standard de la commande vers
            le fichier  crer"""),
    number_of_is('/', 5, 'La bonne rponse contient 5 <em>slash</em>'),
    shell_display,
    ),
    indices=("Il faut rediriger l'entre et la sortie standard",
             "Il ne faut pas oublier d'annuler le <tt>/</tt>",
             ),
    )

dollar_required = require('$',
                     "Je ne vois pas le <tt>$</tt> indiquant la fin de ligne")

add(name="enlever dernier",
    required=["intro", "expreg:ligne de A"],
    question="""Donnez la commande permettant d'enlever
    le dernier caractre de chaque ligne quel qu'il soit.
    <p>
    La commande lit le fichier  transformer sur son entre
    standard et crit le rsultat sur sa sortie standard.
    C'est donc un filtre.""",
    tests=(
    dollar_required,
    require('.',
            """Je ne vois pas le <tt>.</tt> reprsentant un
            caractre quelconque"""),
    shell_good("sed 's/.$//'", dumb_replace=dumb_replace),
    shell_bad("sed 's/.$//g'",
              """Le <tt>g</tt> est inutile car il n'y a qu'une seule
              substitution  faire sur la ligne.""",
              dumb_replace=dumb_replace),
    reject('g', """Pas besoin de l'option <tt>g</tt> car la sustitution
    n'est faite qu'une fois par ligne"""),
    reject('s/.$/$/', "Vous remplacez le dernier caractre par un dollar..."),
    require('//', """Je ne vois pas la squence de caractres,
    indiquant que vous voulez remplacer par rien du tout.
    Je rappelle qu'un espace ce n'est pas rien."""),
    reject('-r',
           """Vous n'avez pas besoin d'expression rgulire tendue,
           enlevez les <tt>-r</tt>"""),
    reject('"', "Le guillemet n'annule pas la signification du dollar"),
    shell_display,
    ),
    )

add(name="TAG XML vides",
    required=["expreg:ligne de A", "enlever dernier"],
    before="""Les fichiers XML contiennent des marqueurs (balises) de la forme :
    <pre>Du &lt;NOM_DU_TAG&gt;texte &lt;b&gt;pouvant&lt;/b&gt; contenir&lt;/NOM_DU_TAG&gt; d'autres &lt;x&gt;&lt;/x&gt; marqueurs.</pre>
    Dans le texte prcdent il y a 3 marqueurs.
    Le marqueur <tt>x</tt> ne contient rien.""",
    question="""Donnez la commande rcrivant sur sa sortie standard
    le fichier XML lu sur son entre standard en liminant
    les marqueurs ne contenant rien.""",
    tests = (
        Expect('sed'),
        Good(Replace(dumb_replace, Shell(
            Equal("sed -r 's/<([^>]*)></\\1>//g'")))),
        Bad(Comment(Replace(dumb_replace, Shell(
            Equal("sed -r 's/<(.*)></\\1>//g'"))),
                    "Le <tt>.*</tt> va <em>avaler</em> le symbole &lt;"),
            ),
        shell_display,
        ),
    )

add(name="ajouter fin",
    required=["enlever dernier"],
    question="""Donnez le filtre ajoutant <tt>toto</tt>
     la fin de chaque ligne lue.""",
    tests=(
    expect('sed'),
    reject('g', """Pas besoin de l'option 'g' puisque la substitution
    n'est possible qu'une fois par ligne"""),
    shell_good("sed 's/$/toto/'", dumb_replace=dumb_replace),
    shell_bad("sed -r 's/(.)$/\\1toto/'",
              "Ceci ne fonctionne pas si la ligne est vide.",
              dumb_replace=dumb_replace),
    shell_bad("sed 's/.*/&toto/'",
              """Ce filtre fontionne, mais il n'est ni le plus court,
              ni le plus performant.
              <p>
              Il suffit de remplacer la fin de ligne par <tt>toto</tt>""",
              dumb_replace=dumb_replace),
    shell_bad("sed 's/.$/&toto/'",
              """Ceci ne fonctionne pas si la ligne est vide."""
              ,dumb_replace=dumb_replace),
    shell_bad("sed 's/$/&toto/'",
              """Savez-vous que le <tt>&amp;</tt> reprsente une chaine
              vide dans la substitution que vous faites&nbsp;?
              Vous pouvez donc faire une commande plus courte."""
              ,dumb_replace=dumb_replace),
    number_of_is('/', 3,"Il n'y a pas le bon nombre de / dans votre commande"),
    dollar_required,
    reject('/.$/', """On ne veut pas remplacer le dernier caractre
    de chaque ligne mais la fin de ligne elle mme"""),
    reject('/toto$/', """Vous ajoutez <tt>toto$</tt>, le dollar
    dans la partie droite n'est pas spcial"""),
    shell_display,
    ),
    indices=(
    """Commencez par rpondre  la question en remplaant le
    dernier caractre par <tt>toto</tt> puis ensuite modifiez
    la commande pour ne plus tenir compte du dernier caractre.
    <em>N'oubliez pas, on veut la commande la plus courte</em>""",    
    ),
    )
    
add(name="enlve mot",
    required=["intro", "expreg:ngation"],
    question="""Quelle filtre (commande lisant son entre standard
    et crivant sur sa sortie standard) permet de remplacer tous
    les caractres du dbut de la ligne jusqu'au premier espace (inclu)
    par rien du tout.
    <p>
    Et ceci pour toutes les lignes.
    <p>
    La ligne <tt>ggg jjj</tt> devient <tt>jjj</tt>
    """,
    tests=(
        shell_good( "sed 's/[^ ]* //'", dumb_replace=dumb_replace),
        reject('g', "On veut faire la substitution qu'une seule fois"),
        reject('/^',"Pas besoin de prciser que c'est le premier de la ligne"),
        shell_display,
        ),
    )



add(name="mot",
    required=["intro", "expreg:ngation"],
    before="""On considrera qu'un mot est un ensemble de
    caractres ne contenant pas d'espace (mais pouvant contenir
    d'autres sparateurs).
    <pre>Je&#9251;suis&#9251;une&#9251;&#9251;phrase.</pre>
    <p>Devient
    <pre>MOT&#9251;MOT&#9251;MOT&#9251;&#9251;MOT</pre>
    """,
    question="""Donner la ligne de commande permettant de remplacer
    tous les mots par le texte suivant&nbsp;: <tt>MOT</tt>.
    On ne changera pas le nombre d'espaces entre les mots.
    """,
    tests=(
    shell_good( ("sed -r 's/[^ ]+/MOT/g'", "sed 's/[^ ][^ ]*/MOT/g'"),
                dumb_replace=dumb_replace,
                ),
    reject("[^ ]*",
              """L'expression <tt>[^ ]*</tt> reprsente des chaines
              vide. Donc MOT risque d'tre inserr entre chaque paire
              d'espace."""),
    shell_bad("sed 's/[^ ]+/MOT/g'",
              """Presque ! Il faut dire  <tt>sed</tt> que vous
              utilisez une expression rgulire tendue""",
              dumb_replace=dumb_replace,
              ),
    reject(".*",
           """<tt>.*</tt> prend la plus grande chaine possible
           il va donc contenir des espaces, ce n'est donc pas un mot"""),
    require("[^ ]",
            """Je ne vois pas le <em>pattern</em> reprsentant
            un caractre qui n'est pas un espace"""),
    shell_require("[^ ]+",
                  """Je ne vois pas le <em>pattern</em> reprsentant
                  un mot sans espace""",
                  dumb_replace=( ("[^ ][^ ]*", "[^ ]+"), ),
                  ),
    require('/g', "Vous ne faites la substitution qu'une fois par ligne..."),
    reject(('a-z','A-Z'), """Un mot n'est pas compos de caractres
    alphabtique mais de caractre qui <b>ne sont pas des espaces</b>
    donc enlevez le <tt>a-z</tt> cela ne sera pas accept."""),
    reject(('/ MOT/', '/MOT /', '/ MOT /'), """Je pense que votre solution ne
    fonctionne pas si le mot est  une extrmit de la ligne..."""),
    shell_display,
    ),
    )

add(name="squencielle",
    required=["intro"],
    question="""Donner la ligne de commande travaillant sur
    l'entre standard qui&nbsp;:
    <ul>
    <li> Remplace tous les <tt>bleu</tt> par <tt>blanc</tt>
    <li> <b>puis</b> remplace tous les <tt>c</tt> minuscule par <tt>C</tt> majuscule
    </ul>
    <p>En lanant une seule fois la commande de remplacement.
    """,
    tests=(
    reject('-r', "Pas besoin d'expression rgulire tendue"),
    require("/bleu/blanc/",
            "Je ne vois pas la substitution de <tt>bleu</tt> en <tt>blanc</tt>"
            ),
    require("/c/C/",
            "Je ne vois pas la substitutions de <tt>c</tt> en <tt>C</tt>"
            ),
    shell_good("sed -e 's/bleu/blanc/g' -e 's/c/C/g'"),
    shell_good("sed 's/bleu/blanc/g;s/c/C/g'",
               """Dans les exercices suivant, la syntaxe avec le <tt>;</tt>
               sera refuse, il est en effet prfrable de faire&nbsp;:
               <pre>sed -e 's/bleu/blanc/g' -e 's/c/C/g'</pre>""",
               dumb_replace=dumb_replace),
    reject((';','|'), "On ne lance <tt>sed</tt> qu'une seule fois"),
    number_of_is('sed', 1, "On utilise <tt>sed</tt> une seule fois"),
    number_of_is('-e', 2,
                 """Vous devez indiquer 2 sustitutions, il devrait
                 y avoir 2 fois l'option <tt>-e</tt>."""),
    number_of_is('/', 6,
                 """Vous devez indiquer 2 sustitutions, il devrait
                 y avoir 6 slashs dans la commande."""),
    number_of_is('/g', 2,
                 """Il devrait y avoir deux fois l'option indiquant
                 que l'on fait toute les substitutions dans la ligne"""),
    shell_display,
    ),
    indices=("Respectez l'ordre de l'nonc sinon le rsultat est diffrent",
             "Il faut indiquer plusieurs fois <tt>-e</tt>",
             ),
    good_answer="""Lancez la commande en tapant 'BLEUbleu'
    et regardez le rsultat, n'est-ce pas trange ?.
    Il faut que vous notiez que les substitutions sont
    faites les unes aprs les autres.""",
    )
    
add(name="inverse",
    required=["intro", "expreg:identique", "mot"],
    before="""On a un fichier contenant dans chaque ligne
    un nom et un prnom.
    On suppose que le SEUL ESPACE de la ligne est entre
    le nom et le prnom.
    <p>
    On suppose aussi qu'il peut arriver que le nom ou le prnom
    soit une chaine vide.
    <pre>Thierry&#9251;EXCOFFIER
&#9251;VANDORPE
Jacques&#9251;
&#9251;</pre>
 <p> Devient
    <pre>EXCOFFIER&#9251;Thierry
VANDORPE&#9251;
&#9251;Jacques
&#9251;</pre>
    """,
    question="""Donnez la ligne de commande utilisant <tt>sed</tt> permettant
    d'inverser le nom et le prnom.
    On laissera <tt>sed</tt> travailler sur l'entre et la sortie standard.
    <p>
    Testez bien la commande avant de proposer la solution,
    comme d'habitude c'est une solution simple qui est demande.
    <p>
    Nom et prnom peuvent contenir n'importe quel caractre (sauf espace).
    """,
    tests=(
    shell_good( ("sed -r 's/([^ ]*) ([^ ]*)/\\2 \\1/'",
                 "sed -r 's/(.*) (.*)/\\2 \\1/'"),
                dumb_replace=dumb_replace,
                ),
    shell_bad( ("sed -r 's/([^ ]+) ([^ ]+)/\\2 \\1/'",
                "sed -r 's/(.+) (.+)/\\2 \\1/'"),
               """Presque juste, mais cela ne fonctionne
               pas si le nom ou le prnom est vide.""",
               dumb_replace=dumb_replace),
    reject('/g', """Le <tt>/g</tt> est inutile puisqu'il n'y a qu'une
    rptition dans la ligne"""),
    number_of_is('(', 2, """On a besoin de faire deux groupes
    pour rpondre  la question"""),
    number_of_is(')', 2, """On a besoin de faire deux groupes
    pour rpondre  la question"""),
    reject('+', "N'oubliez pas que l'on accepte les noms et prnoms vides"),
    require('-r', """Comme on utilise les groupes des expressions rgulires,
    il faut indiquer la bonne option  <tt>sed</tt>"""),
    reject(('^', '$'),
           """<tt>^</tt> et <tt>$</tt> ne sont pas ncessaires pour
           rpondre  cette question"""),
    
    shell_display,
    ),
    indices=("""On doit donner une option particulire  <tt>sed</tt>
    pour qu'il utilise les expressions rgulire tendues""",
             """Ne mettez pas de caractres inutiles comme le <tt>$</tt>
             ou bien le <tt>g</tt>""",
             """Il faut utiliser le tout sauf espace.""",
             """Pensez  utiliser les groupes.""",
             ),
    )

les_modes = """<p>
Comme vous utilisez <tt>mv</tt> le fichier <tt>xxx</tt>
est remplac par le fichier <tt>yyy</tt> qui a t cr avec
les modes par dfaut. Les modes du fichier <tt>xxx</tt>
sont donc perdu.
Pour conserver les modes, il faut faire&nbsp;:
<tt>cat yyy >xxx ; rm yyy</tt>
"""

add(name="fiable",
    required=["dans fichier", "sh:fiable", "deplacer:intro"],
    before="""Vous aurez besoin d'un nom de fichier temporaire,
    prenez <tt>yyy</tt>""",
    question="""Donnez la ligne commande FIABLE prenant le contenu du
    fichier <tt>xxx</tt>, remplaant tous les <tt>a</tt> par des <tt>b</tt>
    et stockant le rsultat dans le fichier <tt>xxx</tt>""",
    tests=(
    require("mv",
            "Vous aviez utilis <tt>mv</tt> pour faire le remplacement fiable"),
    require("&&",
            "Vous aviez utilis <tt>&amp;&amp;</tt> pour faire le remplacement fiable"),
    reject('-i', """L'option <tt>-i</tt> est valide,
    mais vous pouvez rpondre  cette question uniquement
    avec ce que vous savez dj"""),
    reject('rm',
           "Pas besoin de dtruire la destination, <tt>mv</tt> le fait"),
    shell_good(("sed 's/a/b/g' <xxx >yyy && mv yyy xxx",
                "sed 's/a/b/g' xxx >yyy && mv yyy xxx"),
               dumb_replace=dumb_replace),
    shell_good( ("if sed 's/a/b/g' <xxx >yyy ; then mv yyy xxx ; fi",
                 "if sed 's/a/b/g' xxx >yyy ; then mv yyy xxx ; fi",
                 ),
                """Une rponse plus simple&nbsp;:
                <tt>sed -e 's/a/b/g' &lt;xxx &gt;yyy && mv yyy xxx</tt>""",
                dumb_replace=dumb_replace),
    shell_bad( ("sed 's/a/b/g' <xxx >xxx", "sed 's/a/b/g' xxx >xxx"),
               """Cette commande ne fonctionne pas car le fichier est vid
               avant que <tt>sed</tt> ai eu le temps de le lire.
               Vous devez passer par un fichier intermdiaire.""",
               dumb_replace=dumb_replace,
               ),
    shell_bad( ("sed 's/a/b/g' <xxx >yyy ; mv yyy xxx",
                "sed 's/a/b/g' xxx >yyy ; mv yyy xxx",
                ),
               """Cette commande fonctionne, mais elle n'est pas fiable
               car si la substitution est arrte, le disque est plein ou
               s'il y a une erreur dans les arguments de <tt>sed</tt>
               le <tt>mv</tt> aura quand mme lieu et crasera votre fichier.
               Il faut que le <tt>mv</tt> s'excute seulement
               si le remplacement c'est bien pass.
               """,
               dumb_replace=dumb_replace,
               ),
    require(">yyy", """Je ne vois pas comment vous faites pour crer
            le fichier temporaire"""),
    shell_display,
    ),
    indices=("""Utilisez <tt>mv</tt> pour remplacer <tt>xxx</tt> par
    <tt>yyy</tt>""",
             """Vous devez stocker le rsultat de la substitution
             dans <tt>yyy</tt> puis le dplacer  la place <tt>xxx</tt>
             si tout c'est bien pass""",
             ),
    good_answer="""
    Cette solution n'est pas parfaite, en effet les droits d'accs,
    le propritaire, ... du fichier on pu changer en effet ils
    ont t remplac par ceux du fichier <tt>yyy</tt> cr
    avec les valeurs par dfaut.
    <p>
    On peut contourner ce problme en remplaant <tt>mv yyy xxx</tt> par&nbsp;:
    <pre>cat yyy >xxx ; rm yyy</pre>
    L'inconvnient majeur de cette solution est qu'elle n'est plus fiable
    en effet si le disque est plein le fichier <tt>xxx</tt> est tronqu.
    <p>
    Une autre manire de contourner est de faire&nbsp;:
    <pre>chmod --reference=xxx yyy && mv yyy xxx</pre>
    Cette solution est fiable dans le cas ou vous tes propritaire
    du fichier <tt>xxx</tt>
    """,
    )



add(name="remplacer hirarchie",
    required=["fiable", "sh:boucle"],
    before="""Pour remplacer un mot par un autre dans un fichier,
    nous avons fait&nbsp;:
    <pre>sed -e 's/blue/black/g' &lt;xxx &gt;yyy && mv yyy xxx</pre>
    """,
    question="""Quelle ligne de commande permet de remplacer
    <tt>&lt;h1&gt;</tt> par <tt>&lt;h2&gt;</tt>
    dans tous les fichiers se terminant par <tt>.html</tt>
    dans le rpertoire courant.
    <em>Les &lt; et &gt; font partis du texte  remplacer car
    ils encadrent le <em>tag</em> HTML</em>.
    <p>
    Vous utiliserez la variable <tt>I</tt> comme indice de boucle.
    Et <tt>yyy</tt> comme fichier temporaire.
    """,
    tests=(
    reject('find', "Dans le rpertoire courant, pas dans la hirarchie"),
    reject('echo', "Pas besoin de la commande <tt>echo</tt>"),
    shell_good((
    "for I in *.html ; do sed 's/<h1>/<h2>/g' <$I >yyy && mv yyy \"$I\" ; done",
    "for I in *.html ; do sed 's/<h1>/<h2>/g' <\"$I\" >yyy && mv yyy \"$I\" ; done",
    "for I in *.html ; do sed 's/<h1>/<h2>/g' \"$I\" >yyy && mv yyy \"$I\" ; done",
    ),
               dumb_replace=dumb_replace),
    shell_bad(
    "for I in *.html ; do sed 's/<h1>/<h2>/g' <$I >yyy && mv yyy $I ; done",
    """Ne fonctionne pas si un nom de fichier contient un espace (il manque
    les guillemets)""",
    dumb_replace=dumb_replace),
    shell_bad((
    """for I in *.html; do sed 's/<h1>/<h2>/g' >yyy && mv yyy "$I";done""",
    """for I in *.html; do sed 's/<h1>/<h2>/g' >yyy && mv yyy $I;done""",
    ),
    """Comment la commande <tt>sed</tt> sait ce qu'elle doit lire&nbsp;?""",
    dumb_replace=dumb_replace),
    
    reject("xxx",
           "Vous devez remplacer <tt>xxx</tt> par le fichier  transformer"),
    require( ("<h1>", "<h2>"), "Je ne vois pas les textes  remplacer"),
    require( "for", "Vous devez faire une boucle"),
    require( "done", "Et le <tt>do</tt> / <tt>done</tt> ?"),
    require( "&&", "N'oubliez pas, on veut une substitution fiable"),
    require( '$I', "Ou utilisez-vous la variable&nbsp;?"),
    require( '"', """tes-vous sur que cela fonctionne si le nom
    des fichiers contiennent un espace&nbsp;?"""),
    shell_require("<pattern_char>*</pattern_char>.html",
                  """Je ne vois pas le <em>pattern</em> indiquant
                  tous les fichiers <tt>.html</tt>."""),
    reject("ls",
           "On a pas besoin de <tt>ls</tt> pour avoir la liste des fichiers"),
    require('<', 'Comme d\'habitude, on redirige l\'entres standard de <tt>sed</tt>'),
    shell_display,
    ),
    )

add(name="traduit PATH",
    required=["intro", "pipeline:intro", "variable:intro",
              "sh:affiche paramtres spciaux" ],
    question="""Quelle est la commande affichant sur la sortie standard
    le contenu de la variable <tt>PATH</tt> en remplaant chacun
    de deux points '<tt>:</tt>' par un espace.
    <p>
    ATTENTION : pour cette question on n'acceptera pas que des espaces
    multiples disparaissent de la variable <tt>PATH</tt>.
    """,
    tests = (
        Expect('PATH'),
        Expect('$PATH', "On accde au contenu d'une variable avec le..."),
        Expect('"$PATH"',
               "Il faut protger les espaces existants dans la variable"),
        Expect('sed'),
        Expect("/ /", "Vous n'indiquez pas que vous remplacez par un espace"),
        Expect("/g",
               """Indiquez que vous voulez remplacer toutes les occurrences
               et pas seulement la premire de la ligne"""),
        Good(Replace(dumb_replace,Shell(Equal(
            'echo "$PATH" | sed "s/:/ /g"')))),
        shell_display,
        ),
    )

add(name="cherche PATH",
    required=["intro", "traduit PATH", "sh:boucle", "sh:remplacement",
              "lister:affichage long"],
    before="""Quand on lance la commande <tt>sed</tt>,
    le shell recherche la commande dans tous les rpertoires
    indiqus dans la variable <tt>PATH</tt>.
    Il lance la premire commande trouve.
    """,
    question="""On veut excuter la commande <tt>ls -ld</tt>
    sur tous les fichiers de la forme <tt>XXX/sed</tt>
    ou les <tt>XXX</tt> sont les noms des rpertoires indiqus dans
    la variable <tt>PATH</tt>.
    Pour faire cela, vous avez besoin de&nbsp;:
    <ul>
    <li> De la boucle pour pour parcourir les noms de rpertoire.
    <li> De remplacer une commande par le rsultat de son excution.
    Ceci pour gnrer la liste des indices de la boucle
    car les <tt>:</tt> qui sont dans <tt>PATH</tt>
    ne sont pas des sparateurs pour le shell.
    <li> Du corps de boucle pour faire le <tt>ls -ld</tt> en ajoutant
    <tt>sed</tt> au rpertoire qui est dans l'indice.
    </ul>
    Vous utiliserez <tt>I</tt> comme indice de boucle.
    """,
    tests=(
    require("I", "Vous n'avez pas utilis la variable <tt>I</tt>"),
    require("PATH", "Vous n'avez pas utilis la variable <tt>PATH</tt>"),
    require("/ /", "Vous n'indiquez pas que vous remplacez par un espace"),
    require("/g",
           """Indiquez que vous voulez remplacer toutes les occurrences
           et pas seulement la premire de la ligne"""),
    require("/sed",
            """On veut les informations sur le fichier <tt>sed</tt>
            dans chacun des rpertoires"""),
    shell_good(
    "for I in $(echo \"$PATH\" | sed 's/:/ /g') ; do ls -ld \"$I\"/sed ; done",
    dumb_replace=dumb_replace),
    shell_good(
        ("for I in $(echo $PATH | sed 's/:/ /g'); do ls -ld \"$I\"/sed ; done",
         "for I in $(echo \"$PATH\" | sed 's/:/ /g'); do ls -ld $I/sed ; done",
         "for I in $(echo $PATH | sed 's/:/ /g'); do ls -ld $I/sed ; done",
         ),
        """Il est fortement recommand de mettre systmatiquement
        des guillemets autour des accs aux variables&nbsp;:
        <tt>"$PATH"</tt> au lieu de <tt>$PATH</tt>""",
        dumb_replace=dumb_replace),
    require("|", """Il faut faire un pipeline avec <tt>echo</tt>
    et <tt>sed</tt> pour obtenir le contenu de la variable <tt>PATH</tt>
    avec des espaces  la place des ':'"""),
    shell_require(">PATH</variable>",
                  "Vous n'utilisez pas le contenu de la variable <tt>PATH</tt>"),
    shell_require(">I</variable>",
                  "Vous n'utilisez pas le contenu de <tt>I</tt>"),
    shell_require("<replacement",
                  """Je ne vois pas de 'remplacement'.
                  La commande <tt>for</tt> a besoin du rsultat
                  du pipeline pour avoir la liste des paramtres."""), 
    shell_display,
    ),
    good_answer="""La solution que vous avez propose est en effet
    la plus simple.
    Mais elle montre l'norme limitation du shell&nbsp;:
    il peut faire n'importe quoi ds que des variables
    contiennent des espaces.
    <p>
    Si la variable <tt>PATH</tt> contient des noms de rpertoires
    avec des espaces votre script ne fonctionnera pas.""",
    )


add(name="garde slash",
    required=["mot", "slash"],
    question="""Donner la commande lisant son entre standard
    et affichant sur sa sortie standard que les <tt>/</tt>.
    <p>
    Si l'on met <tt>/usr/include</tt> dans l'entre standard,
    il en sort : <tt>//</tt>""",
    tests=(
    reject('*', "Pourquoi utilisez-vous une rptition de caractres&nbsp;?"),
    reject('-r', "Pas besoin d'expression rgulire tendue"),
    reject('+', """Le <tt>+</tt> fonctionne avec les expressions
    rgulires tendues. Il n'est pas ncessaire de l'utiliser dans
    cet exemple car on remplace par rien et cela ne gne pas
    de remplacer rien par rien."""),
    reject('/^', """Les caractres  substituer peuvent tre partout
    dans la ligne.
    Pourquoi dites-vous qu'ils sont en dbut de ligne&nbsp;?"""),
    reject('!', """Le ! est pour les <em>pattern</em> du shell, pas
    les expressions rgulires"""),
    shell_good(r"sed 's/[^/]//g'", dumb_replace=dumb_replace),
    reject( "[^\/]",
                """Les crochets annulent la signification de l'antislash
                et du slash. Entre crochet, le slash et l'antislash
                sont des caractres sans signification particulire.
                Vous n'avez donc pas besoin de les protger par
                un antislash."""
               ),
    require('//', """Je ne vois pas la liste de caractres indiquant que vous
    voulez remplacer par rien du tout les caractres autres."""),
    require('/g', """Il manque l'option de substitution indiquant qu'il
    y a plusieurs substitutions  faire sur la ligne"""),
    require('^', """On a besoin de la ngation pour indiquer 'tout sauf'
    pour un ensemble de caractres."""),
    #    answer_length_is(15, 
    shell_display,
    ),
    )
    
add(name="annule racine",
    required=["mot", "slash"],
    question="""Donner la commande lisant son entre standard
    et affichant sur sa sortie standard ce qu'elle a lu,
     l'exception de la ligne dont le contenu exacte est&nbsp;: <tt>/</tt>
	(un caractre unique)
    qui est remplac par une ligne vide.
    <p>
    <tt>usr/include</tt> est inchang<br>
    <tt>/</tt> disparait, il reste une ligne vide""",
    tests=(
    expect('sed'),
    reject("/g", """Il n'y a qu'une seule substitution  faire
    sur la ligne, l'option de remplacement 'g' est en trop."""),
    reject('-r', "On a pas besoin d'expression rgulire tendue"),
    shell_good(r"sed 's/^\/$//'", dumb_replace=dumb_replace),
    reject( "[/]",
                """Il y a plus simple que les crochets pour
                annuler la signification du slash."""),
    require(('^','$'), """Pour indiquer que <tt>/</tt> est seul sur
    la ligne, il faut indiquer que le <em>pattern</em> reprsente
    une ligne complte et qu'il va donc du dbut jusqu' la fin"""),
    number_of_is('/', 4, "Il n'y a pas le bon nombre de /"),
    require('//', """Je ne vois pas la squence de caractres indiquant
    que vous remplacer le texte trouv par rien"""),
    reject('[', "Pourquoi il y a un crochet dans votre rponse&nbsp;?"),
    reject('*', "Pourquoi il y a une toile dans votre rponse&nbsp;?"),
    shell_display,
    ),
    indices=("""<tt>^</tt> reprsente le dbut de ligne et
    <tt>$</tt> reprsente la fin de ligne""",),
    )

comm = """Donnez la ligne de commande affichant votre profondeur
    dans la hirarchie de fichier.    
    <p>
    Si vous tes dans <tt>/</tt> cela doit vous afficher 1.<br>
    Si vous tes dans <tt>/etc</tt> cela doit vous afficher 2.<br>
    Si vous tes dans <tt>/usr/include</tt> cela doit vous afficher 3.
    <p>
    Inspirez-vous des questions auxquelles vous avez dj rpondu
    et n'utilisez pas le <tt>|</tt> des expressions rgulires.
    <p>
    """

add(name="profondeur wc",
    required=["compte:echo", "sh:directory courant",
              "intro", "annule racine", "garde slash", "pipeline:intro",
              "squencielle"
              ],
    question=comm + """<b>Vous utiliserez la commande <tt>wc</tt>
    pour calculer la longueur</b>""",
    tests=(
    reject('echo', "Pas besoin de <tt>echo</tt> pour cette question."),
    shell_good(r'''pwd | sed -e 's/[^/]//g' -e 's/^\/$//'  | wc -c'''),
    shell_good(r'''pwd | sed -e 's/^\/$//' -e 's/[^/]//g' | wc -c'''),
    shell_good((r'''pwd | sed   's/^\/$//' -e 's/[^/]//g' | wc -c''',
                r'''pwd | sed   's/[^/]//g' -e  's/^\/$//'  | wc -c'''
                ),
               "Gnralement on met l'option <tt>-e</tt> deux fois"),
    shell_bad(r'''pwd | sed -e 's/[^/]//g' | wc -c''',
              "Ne fonctionne pas pour la racine"),
    number_of_is("sed", 1,
                 "On lance la commande <tt>sed</tt> une seule fois"),
    reject(r'[^\/]', "Pas besoin d'antislash avant le / entre les crochets"),
    reject(('*','+'), """Pas besoin de '*' ou '+' pour cette question"""),
    require('s/^\/$//', """Je ne vois pas la substitution qui enlve
    les / qui sont tout seul sur une ligne (cas particulier de la racine)"""),
    require('s/[^/]//g', """Je ne vois pas la substitution qui enlve
    tous les caractres qui ne sont pas des slash."""),
    reject('-m', """Utilisez plutt <tt>-c</tt>  la place de <tt>-m</tt>"""),
    shell_bad("pwd | sed -e 's/[^/]//g' -e 's/^\/$//g' | wc -c",
              """Ne fonctionne pas dans le premier niveau (<tt>/etc</tt>
              par exemple.)""",
              ),
    reject('s/^\/$//g', """ quoi sert l'option <tt>g</tt> s'il n'y
    a qu'une seule substitution  faire sur la ligne&nbsp;?"""),
    number_of_is('|', 2, "Il y a 3 commandes  faire donc il faut 2 pipe."),
    reject(('-m', '--chars'),
           """Comptez le nombre d'octets plutt que de caractres."""),
    shell_display,
    ),
    indices=( """Il faut compter le nombre de <tt>/</tt>""",
              """La racine (<tt>/</tt> tout seul) est un cas particulier,
              cela devrait tre une chaine vide.""",
              ),
    )

add(name="profondeur expr",
    required=["profondeur wc", "sh:remplacement", "calculer:longueur"],
    question=comm + """<b>Vous utiliserez la commande <tt>expr</tt>
    pour calculer la longueur</b>""",
    tests=(
    reject('/.$/', '''Votre solution est trop astucieuse pour que le systme
    puisse la tester.
    Utilisez ce qui est propos dans les prrequis'''),
    shell_good(r'''expr length "$(pwd | sed -e 's/^\/$//' -e 's/[^/]//g')" + 1'''),
    shell_good(r'''expr length $(pwd | sed -e 's/^\/$//' -e 's/[^/]//g') + 1'''),
    shell_good(r'''expr 1 + length "$(pwd | sed -e 's/^\/$//' -e 's/[^/]//g')"'''),
    shell_good(r'''expr 1 + length $(pwd | sed -e 's/^\/$//' -e 's/[^/]//g')'''),
    shell_good(r'''expr length "$(pwd | sed -e 's/[^/]//g' -e 's/^\/$//' )" + 1'''),
    shell_good(r'''expr length $(pwd | sed -e 's/[^/]//g' -e 's/^\/$//' ) + 1'''),
    shell_good(r'''expr 1 + length "$(pwd | sed -e 's/[^/]//g' -e 's/^\/$//' )"'''),
    shell_good(r'''expr 1 + length $(pwd | sed -e 's/[^/]//g' -e 's/^\/$//' )'''),
    reject(r'[^\/]', "Pas besoin d'antislash avant le / entre les crochets"),
    reject('=', "Vous n'avez pas besoin de =&nbsp;!"),
    expect('pwd'),
    expect("length"),
    require('1', "Il faut ajouter 1 au nombre de <em>slash</em> prsent"),
    shell_display,               
    ),
    )

add(name="basename",
    required=["sh:boucle", "pattern:.c et .h",
              "sh:remplacement", "deplacer:intro", "slash",
              "enlve commentaires"],
    before="""La commande standard <tt>basename NomFichier Extension</tt>
    fait l'quivalent de&nbsp;:
    <pre>echo NomFichier | sed -e 's/.*\\///' -e 's/Extension$//'</pre>
    Elle enlve donc le nom du rpertoire et l'extension.
    <tt>basename /usr/include/stdio.h .h</tt> donne donc <tt>stdio</tt>
    """,
    question="""Pour tous les fichiers <tt>.c</tt> du rpertoire
    courant renommez les fichiers en remplaant le <tt>.c</tt>
    par <tt>.xc</tt>
    <p>
    Vous utiliserez la variable shell <tt>I</tt> comme
    indice de boucle.
    """,
    tests=(
    reject("sed",
           """Utilisez la commande <tt>basename</tt>, pas <tt>sed</tt>
           qui tait l seulement pour vous expliquer."""),
    reject('echo', "On n'a pas besoin de la commande <tt>echo</tt>"),
    require("mv", "On renomme avec la commande <tt>mv</tt>"),
    require("for", "On doit faire une boucle <tt>for</tt>"),
    require("basename", "On veut utiliser la commande <tt>basename</tt>"),
    shell_require("<replacement",
                  """Je ne vois pas de 'remplacement'.
                  La commande <tt>mv</tt> a besoin du rsultat
                  de la commande <tt>basename</tt> comme
                  paramtre"""), 
    shell_good('for I in *.c ; do mv "$I" "$(basename "$I" c)xc" ; done'),
    shell_bad(('for I in *.c ; do mv "$I" $(basename "$I" c)xc ; done',
               'for I in *.c ; do mv "$I" "$(basename $I c)"xc ; done'),
              """Presque&nbsp;! Il manque encore des guillemets."""),
    reject(" c",
           """Le fichier <tt>toc</tt> va tre renomm en <tt>tocpp</tt>,
           mettez le point."""),
    require(".xc", "On veut ajouter le suffixe <tt>.xc</tt>"),
    require('"',
            """N'oubliez pas le cas o le nom du fichier contient un espace,
            il faut protger l'accs au variables."""),
    reject(" $I ",
           "Tous les accs aux variables doivent tre entre guillemets"),
    shell_good('for I in *.c ; do mv "$I" "$(basename "$I" .c).xc" ; done'),
    shell_bad(('for I in *.c ; do mv "$I" $(basename "$I" .c).xc ; done',
               'for I in *.c ; do mv "$I" "$(basename $I .c)".xc ; done'),
              """Presque&nbsp;! Il manque encore des guillemets."""),
    shell_require("<argument>.c</argument>",
                  """Il manque un argument  <tt>basename</tt> pour
                  lui dire d'enlever le <tt>.c</tt> de <tt>$I</tt>"""),
    require('*', """O est l'toile indiquant que vous voulez
    tous les <tt>.c</tt>"""),
    shell_require("<pattern_char>*</pattern_char>",
                  """Il ne faut pas protger l'toile. Sinon est est
                  considre comme une vrai toile et non
                  un lment de <em>pattern</em>"""),
    require("I", "Je ne vois pas le nom de la variable qui est l'indice"),
    shell_display,               
    ),
    good_answer="""
    Attention, je ne garantis pas que cette commande s'valuera
    correctement avec tous les interprteurs shells.
    Il y a en effet une imbrication de guillemets dans cette commande.""",
    )

add(name="enlve sans point",
    required=['intro', 'expreg:un spcial'],
    question="""Quelle ligne de commande filtre l'entre standard en remplaant
    sur chacune des lignes le dernier '.' et ce qui est  sa gauche par rien.
    S'il n'y a pas de point rien n'est fait.
    <p>
    Par exemple :
    <table><tr><th>Entre<th>Sortie</tr>
    <tr><td><pre>toto
toto.c
truc
truc.tar
truc.tar.gz</pre>
<td><pre>toto
c
truc
tar
gz</pre></tr></table>""",
    tests = (
        Expect('sed'),
        Expect('*', """Le remplacement ce fait avec des suites de plusieurs
        caractres. Votre rponse ne contient rien dans ce sens..."""),
        Reject('/g', "Une seule substitution est ncessaire"),
        
        Good(Replace(dumb_replace,Shell(
            Equal("sed 's/.*\\.//'") | Equal("sed 's/.*[.]//'") ))),
        shell_display,
        ),
    )

add(name="enlve point gauche",
    required=['intro','expreg:ngation', 'expreg:ligne de A'],
    question="""Quelle ligne de commande filtre l'entre standard en remplaant
    les lignes ne contenant pas le caractre '.' par une ligne vide.
    <p>
    Par exemple :
    <table><tr><th>Entre<th>Sortie</tr>
    <tr><td><pre>toto
toto.c
truc
truc.tar
truc.tar.gz</pre>
<td><pre>&nbsp;
toto.c

truc.tar
truc.tar.gz</pre></tr></table>""",
    tests = (
        Expect('sed'),
        Bad(Comment(Replace(dumb_replace + (('/g','/'),),Shell(
            Equal("sed 's/[^.]//'") | Equal("sed 's/[^.]*//'")
            )),
                    """Vous enlevez les caractres qui ne sont pas
                    des points dans la ligne !""")),
        Expect('^',
               """Vous n'avez pas indiqu que l'expression commenait en dbut
               de ligne"""),
        Expect('$',
               """Vous n'avez pas indiqu que l'expression finissait en fin
               de ligne"""),
        Expect('*', """Le remplacement ce fait avec des suites de plusieurs
        caractres. Votre rponse ne contient rien dans ce sens..."""),
        
        Bad(Comment(Replace(dumb_replace,Shell(Equal("sed 's/^[^.]*$//g'"))),
                    "Une seule substitution est ncessaire")),
        Good(Replace(dumb_replace,Shell(Equal("sed 's/^[^.]*$//'")))),
        shell_display,
        ),
    )

add(name="extensions",
    required=["enlve sans point", "enlve point gauche", "squencielle"],
    question="""Combiner CORRECTEMENT les questions auquelles vous avez
    dj rpondu pour obtenir le rsultat suivant :
<table><tr><th>Entre<th>Sortie</tr>
    <tr><td><pre>toto
toto.c
truc
truc.tar
truc.tar.gz</pre>
<td><pre>&nbsp;
c

tar
gz</pre></tr></table>""",
    tests = (
        Good(Shell(Equal("sed -e 's/^[^.]*$//' -e 's/.*\\.//'"))),
        shell_display,
        ),
    )

add(name="liste extensions",
    required=["extensions", "trier:unique"],
    question="""Donnez la ligne de commande listant les extensions de fichiers
    utilises dans le rpertoire courant.
    Ne rptez pas les mmes extensions plusieurs fois.
    <p>
    Ce n'est pas grave si une ligne vide est affiche.""",
    tests = (
        Good(Shell(
            Equal("ls | sed -e 's/^[^.]*$//' -e 's/.*\\.//' | sort -u")
            | Equal("ls | sed -e 's/^[^.]*$//' -e 's/.*[.]//' | sort -u")
            )),
        ),
    default_answer = "ls | ",
    )

from awk import awk_compte

add(name="compter extensions",
    required=["extensions", "awk:compte"],
    question="""Donnez la commande qui pour chaque extension de fichier
    trouve dans le rpertoire courant,
    indique pour combien de fichiers elle s'applique.
    <p>
    Recopiez exactement les diffrents morceaux de commande sinon
    la rponse ne sera pas accepte.
    """,
    default_answer = "ls | ",
    tests = (
        Good(Shell(
            Equal("ls | sed -e 's/^[^.]*$//' -e 's/.*\\.//' | %s" % awk_compte)
            | Equal("ls | sed -e 's/^[^.]*$//' -e 's/.*[.]//' | %s"%awk_compte)
            )),
        ),
    )



# Pas possible
##add(name='balise HTML',
##    required=[],
##    question="""Quelle commande tapez-vous pour copier le
##    fichier <tt>xxx.html</tt> sous le nom  <tt>yyy.html</tt>
##    en passant en majuscules les balises HTML.
##    <table>
##    <tbody>
##    <tr>
##    <th>Avant</th>
##    <td>
##    &lt;p&gt;La traduction de &lt;span lang=\"en\"&gt;directory&lt;/span&gt;
##    est rpertoire.&lt;/p&gt;
##    </td>
##    </tr>
##    <tr>
##    <th>Aprs</th>
##    <td>
##    &lt;P&gt;La traduction de &lt;SPAN lang=\"en\"&gt;directory&lt;/SPAN&gt;
##    est rpertoire..&lt;/P&gt;
##    </td>
##    </tr>
##    </tbody>
##    </table>
##    """,
##    tests=(
##    good("sed xxx.html >yyy.html 's/<")
##    ),
    
##    )

# Faire le 2> /dev/null
# Faire la version avec while

              
    
    
    
    
